#define maxN 100000
#define maxM 100000

#include <stdio.h>
#include <cstdlib>
#include <vector>

using namespace std;

const int OK = 0;
const int FAIL = 3;
const int PE = 4;
const int WA = 5;

FILE *fi, *fo, *fa, *fr;
bool report;
int N, M, e[maxM][2], ans;
vector<char> cmp, tmp;
int permutation[maxN], order[maxN], left[maxN + 1], right[maxN + 1], first[maxN + 1], second[maxN + 1];
bool index[maxN + 1];

void quit(int exitCode, char *message)
{
    if (report)
    {
        fprintf(fr, "%s\n", message);
        fclose(fr);
    }
    else
        printf("%s\n", message);
    fclose(fi);
    fclose(fo);
    fclose(fa);
    exit(exitCode);
}

bool compareByCharacters()
{
    fseek(fo, 0, SEEK_SET);
    char c;
    for (int i = 0; i < cmp.size(); i++)
        if (fscanf(fo, "%c", &c) < 1 || c != cmp[i])
            return false;
    if (fscanf(fo, "%c", &c) >= 1)
        return false;
    return true;
}

int main(int argc, char *argv[])
{
    if (argc < 4)
    {
        printf("usage: permutation_checker.exe INPUT-FILE CONTESTANT-OUTPUT-FILE AUTHOR-OUTPUT-FILE [REPORT-FILE]\n");
        return 1;
    }
    fi = fopen(argv[1], "r");
    fo = fopen(argv[2], "r");
    fa = fopen(argv[3], "r");
    if (argc >= 5)
    {
        fr = fopen(argv[4], "a");
        report = true;
    } else
        report = false;

    fscanf(fi, "%d %d", &N, &M);
    for (int i = 0; i < M; i++)
        fscanf(fi, "%d %d", &e[i][0], &e[i][1]);
    fscanf(fa, "%d", &ans);

    if (fscanf(fo, "%d", &permutation[0]) < 1)
        quit(WA, "The output is incorrect (1)");
    if (permutation[0] == 0)
    {
        cmp.push_back('0');
        cmp.push_back('\n');
        //if (! compareByCharacters())
        //    quit(WA, "The output is incorrect (2)");
        if (ans != 0)
            quit(WA, "Wrong answer");
        else
            quit(OK, "OK");
    } else
    {
        for (int i = 1; i < N; i++)
            if (fscanf(fo, "%d", &permutation[i]) < 1)
                quit(WA, "The output is incorrect (3)");
        for (int i = 0; i < N - 1; i++)
            if (fscanf(fo, "%d", &order[i]) < 1)
                quit(WA, "The output is incorrect (4)");

        for (int i = 1; i <= N; i++)
            index[i] = false;
        for (int i = 0; i < N; i++)
        {
            if (permutation[i] < 1)
                quit(WA, "The output is incorrect: some of the permutation numbers are less than 1");
            if (permutation[i] > N)
                quit(WA, "The output is incorrect: some of the permutation numbers are greater than N");
            if (index[permutation[i]])
                quit(WA, "The output is incorrect: some of the permutation numbers are listed twice");
            index[permutation[i]] = true;
        }

        for (int i = 1; i <= N; i++)
            index[i] = false;
        for (int i = 0; i < N - 1; i++)
        {
            if (order[i] < 1)
                quit(WA, "The output is incorrect: some of the order numbers are less than 1");
            if (order[i] > N)
                quit(WA, "The output is incorrect: some of the order numbers are greater than N");
            if (index[order[i]])
                quit(WA, "The output is incorrect: some of the order numbers are listed twice");
            index[order[i]] = true;
        }

        for (int i = 0; i < N; i++)
        {
            tmp.clear();
            int cur = permutation[i];
            while (cur > 0)
            {
                tmp.push_back((cur % 10) + '0');
                cur /= 10;
            }
            for (int j = tmp.size() - 1; j >= 0; j--)
                cmp.push_back(tmp[j]);
            cmp.push_back(i < N - 1 ? ' ' : '\n');
        }

        for (int i = 0; i < N - 1; i++)
        {
            tmp.clear();
            int cur = order[i];
            while (cur > 0)
            {
                tmp.push_back((cur % 10) + '0');
                cur /= 10;
            }
            for (int j = tmp.size() - 1; j >= 0; j--)
                cmp.push_back(tmp[j]);
            cmp.push_back(i < N - 2 ? ' ' : '\n');
        }

        //if (! compareByCharacters())
        //    quit(WA, "The output is incorrect (5)");

        if (ans == 0)
            quit(WA, "Wrong answer");

        left[permutation[0]] = -1;
        for (int i = 1; i < N; i++)
            left[permutation[i]] = permutation[i - 1];

        right[permutation[N - 1]] = -1;
        for (int i = 0; i < N - 1; i++)
            right[permutation[i]] = permutation[i + 1];

        for (int i = 1; i <= N; i++)
            first[i] = second[i] = -1;

        for (int i = 0; i < N - 1; i++)
        {
            int cur = order[i];
            if (left[cur] != -1)
            {
                first[cur] = left[cur];
                right[left[cur]] = right[cur];
            }
            if (right[cur] != -1)
            {
                second[cur] = right[cur];
                left[right[cur]] = left[cur];
            }
        }

        for (int i = 1; i <= N; i++)
            index[i] = false;

        for (int i = 0; i < M; i++)
        {
            if (first[e[i][1]] == e[i][0])
                index[e[i][1]] = true;
            if (second[e[i][0]] == e[i][1])
                index[e[i][0]] = true;
        }

        int false_cnt = 0;
        for (int i = 1; i <= N; i++)
            if (! index[i])
                false_cnt++;

        if (false_cnt > 1)
            quit(WA, "Wrong answer");

        quit(OK, "OK");
    }

    quit(FAIL, "Checker error");    // Shouldn't happen
}
