#include <stdio.h>
#include <cstdlib>

#include <fstream>
#include <string.h>
#include <vector>

using namespace std;

const int OK = 0;
const int FAIL = 3;
const int PE = 4;
const int WA = 5;

FILE *fi, *fo, *fa, *fr;
bool report;

vector<string> s, r, in, out;
int maxTime = 1000, maxProgramLength = 100, maxSearchLength = 10, maxReplacementLength = 10;

int KMP(string S, string K)
{
    vector<int> T(K.size() + 1, -1);

	for (int i = 1; i <= K.size(); i++)
	{
		int pos = T[i - 1];
		while (pos != -1 && K[pos] != K[i - 1])
            pos = T[pos];
		T[i] = pos + 1;
	}

	int sp = 0, kp = 0;
	while(sp < S.size())
	{
		while (kp != -1 && (kp == K.size() || K[kp] != S[sp]))
            kp = T[kp];
		kp++;
		sp++;
		if (kp == K.size())
            return sp - K.size();
	}

	return -1;
}

int automaton(vector<string> s, vector<string> r, string in, int limit, string &out, FILE* fdebug)
{
    out = in;
    int time = 0;
    while(true)
    {
        if (fdebug != NULL)
            fprintf(fdebug, "%s\n", out.c_str());
        bool replaced = false;
        for (int i = 0; i < s.size(); i++)
        {
            int pos = KMP(out, s[i]);
            if (pos >= 0)
            {
                time++;
                if (time > limit)
                    return -1;
                out = out.substr(0, pos) + r[i] + out.substr(pos + s[i].size());
                replaced = true;
                break;
            }
        }
        if (! replaced)
            return time;
    }
}

void quit(int exitCode, char *message)
{
    if (report)
    {
        fprintf(fr, "%s\n", message);
        fclose(fr);
    }
    else
        printf("%s\n", message);
    fclose(fi);
    fclose(fo);
    fclose(fa);
    exit(exitCode);
}

int main(int argc, char *argv[])
{
    if (argc < 4)
    {
        printf("usage: checker.exe INPUT-FILE CONTESTANT-OUTPUT-FILE AUTHOR-OUTPUT-FILE [REPORT-FILE]\n");
        return 1;
    }
    fi = fopen(argv[1], "r");
    fo = fopen(argv[2], "r");
    fa = fopen(argv[3], "r");
    if (argc >= 5)
    {
        fr = fopen(argv[4], "a");
        report = true;
    } else
        report = false;

    char c;
    int mode = 0; // Looking for a new rule
    int emptyLines = 0, programLength = 0;
    string searchString = "", replacementString = "";
    while (true)
    {
        int readCount = fscanf(fo, "%c", &c);
        if (readCount <= 0 || c == '\r' || c == '\n')
        {
            if (mode != 0)
            {
                s.push_back(searchString);
                r.push_back(replacementString);
                searchString = "";
                replacementString = "";
                mode = 0; // Looking for a new rule
            } else if (readCount > 0 && c == '\n')
            {
                emptyLines++;
                if (emptyLines > 10)
                {
                    quit(WA, "(1)");
                }
            }
        } else if (c == ' ')
        {
            if (mode != 1)
            {
                quit(WA, "(2)");
            }
            mode = 2; // Expecting the replacement string
        } else if (c == '+' || c >= '0' && c <= '9' || c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z')
        {
            if (mode == 0)
            {
                searchString += c;
                mode = 1; // Reading search string
                emptyLines = 0;
                programLength++;
                if (programLength > maxProgramLength)
                {
                    quit(WA, "(3)");
                }
            } else if (mode == 1)
            {
                searchString += c;
                if (searchString.size() > maxSearchLength)
                {
                    quit(WA, "(4)");
                }
            } else if (mode == 2)
            {
                replacementString += c;
                mode = 3; // Reading replacement string
            } else if (mode == 3)
            {
                replacementString += c;
                if (replacementString.size() > maxReplacementLength)
                {
                    quit(WA, "(5)");
                }
            }
        } else
        {
            quit(WA, "(6)");
        }
        if (readCount <= 0)
            break;
    }

    in.push_back("1+2");
    out.push_back("3");

    in.push_back("37+8");
    out.push_back("45");

    in.push_back("50+61");
    out.push_back("111");

    in.push_back("88+96");
    out.push_back("184");

    in.push_back("100+100");
    out.push_back("200");

    for (int i = 0; i < in.size(); i++)
    {
        string outString;
        if (automaton(s, r, in[i], maxTime, outString, NULL) == -1)
            quit(WA, "(7)");
        if (outString != out[i])
            quit(WA, "(8)");
    }

    quit(OK, "OK");
}
